package main

import (
	"log"

	"git.joco.dk/sng/fermentord/internal/controllers"
	"git.joco.dk/sng/fermentord/pkg/temperature"
	"github.com/getsentry/sentry-go"
	"github.com/spf13/viper"
)

type configuration struct {
	controllers.ControllerConfig

	NATS struct {
		URL     string `mapstructure:"url"`
		Stream  string `mapstructure:"stream"`
		Subject struct {
			Temp  string `mapstructure:"temp"`
			State string `mapstructure:"state"`
		} `mapstructure:"subject"`
	} `mapstructure:"nats"`

	HTTP struct {
		Port int16 `mapstructure:"port"`
	} `mapstructure:"http"`

	DataPath string `mapstructure:"data_path"`
}

func loadConfiguration() *configuration {
	viper.SetDefault("data_path", "./data.txt")
	viper.SetDefault("nats.url", "nats.service.consul")
	viper.SetDefault("nats.stream", "FERMENTOR")
	viper.SetDefault("nats.subject.state", "FERMENTOR.state")
	viper.SetDefault("nats.subject.temp", "FERMENTOR.temp")
	viper.SetDefault("http.port", 8000)
	viper.SetDefault("pid.kp", 2.0)
	viper.SetDefault("pid.ki", 0.0001)
	viper.SetDefault("pid.kd", 2.0)

	viper.AddConfigPath("/etc")
	viper.AddConfigPath("/usr/local/etc")
	viper.AddConfigPath(".")
	viper.SetConfigName("fermentord")
	viper.SetConfigType("hcl")
	viper.AutomaticEnv()

	if err := viper.ReadInConfig(); err != nil {
		log.Printf("Error loading configuration: %v", err)
	}

	config := &configuration{}
	if err := viper.Unmarshal(&config); err != nil {
		sentry.CaptureException(err)
		log.Fatal(err)
	}

	return config
}

func reloadConfiguration(config *configuration, ctrl *controllers.ChamberController) {
	temperature.ConfigUpdates <- []string{
		config.Sensor.Ambient,
		config.Sensor.Chamber,
		config.Sensor.Wort,
	}

	ctrl.ConfigUpdates <- config.ControllerConfig
}
